/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language5a.*;

/**
   This program parses and then evaluates strings from Language_5.
*/
public class Language_5a_Examples
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      EvaluateAST_no_checking.DEBUG = 1;

      String[] programs =
      {
         // 0
         String.join("\n",
         "(prog",
         "  (fun f (lambda x (* x y)))",  // y is not defined
         "  (var z (apply g a)))"),       // g is not defined, a is not defined

         // 1
         String.join("\n",
         "(prog",
         "  (var w 10)",
         "  (fun f (lambda x y x (+ x y x)))",    // parameter x is repeated
         "  (var z (apply f (+ w x y) 10 20)))"), // x and y are not defined

         // 2
         String.join("\n",
         "(prog",
         "  (var c 1)",
         "  (fun f (lambda x (+ x y)))",  // error, undefined variable y
         "  (fun g (lambda x (+ x c)))",  // no error, c is a global variable
         "  (apply f 12)",
         "  (apply g 12))"),

         // 3
         String.join("\n",
         "(prog",
         "  (var w 2)",
         "  (fun f (lambda x (* x x w)))",
         "  (fun f (lambda y (+ y y w)))",  // f is redefined
         "  (var z (apply f (apply f 5))))"),

         // 4
         String.join("\n",
         "(prog",
         "  (fun f (lambda x (* x x)))",
         "  (var h f)",
         "  (apply h 1 2)",   // wrong number of arguments for h
         "  (apply f 1 2))"), // wrong number of arguments for f

         // 5
         String.join("\n",
         "(prog",
         "  (fun f (lambda x (* x x)))",
         "  (var h f)",
         "  (apply h 1 2))"), // wrong number of arguments for h

         // 6
         String.join("\n",
         "(prog",
         "  (fun f (lambda 5))",          // constant function
         "  (fun g (lambda 5 6))",        // invalid function
         "  (fun h (lammbda x (* x x)))", // invalid function
         "  (apply f))"),

         // 7
         String.join("\n",
         "(prog",
         "  (fun f (lambda true false))",
         "  (fun g (lambda if x x))",
         "  (fun h (lambda if x y))",
         "  (apply f))"),
      };

      int i = 0;
      for (final String str : programs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the program.
         try
         {
            final Tree<Token> ast = Parse2AST.parse( str );

            // Print the AST as an S-expression
            //System.out.println( EvaluateAST.toString(ast) + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.println("// Unparsed program: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_5a_Examples.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            // Check the AST.
            AnalyzeAST.check( ast );

            // Run the program (interpret the AST).
            final Value value = EvaluateAST_no_checking.eval( ast );
            if (EvaluateAST_no_checking.DEBUG > 0)
            {
               System.out.println("result = " + value + "\n" );
            }
            else
            {
               System.out.println("result = " + value.toSimpleString() + "\n" );
            }
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (AnalysisError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
