/*

*/

package language5a;

import tree.Tree;
import tree.LinkedTree;
import tokenizer.Token;

/**
   This class is a kind of "tagged union".
   <p>
   Each instance of this class holds a value for our simple language.
   But our language has two kinds of values, ints and booleans. So
   each instance of this class holds both an int value and a boolean
   value. The "tag" field is used to determine which of the two
   values that an instance holds, the int or the boolean, is valid.
   <p>
   See <a href="https://en.wikipedia.org/wiki/Tagged_union" target="_top">
                https://en.wikipedia.org/wiki/Tagged_union</a>
*/
public class Value
{
   public final String tag;     // either "int" or "bool" or "lambda"
   public final int valueI;
   public boolean valueB;
   public final Tree<Token> valueL;

   public static final String INT_TAG = "int";
   public static final String BOOL_TAG = "bool";
   public static final String LAMBDA_TAG = "lambda";

   /**
      Construct a {@code Value} object that holds an int value.

      @param value  the integer value for this {@code Value} object
   */
   public Value(int value)
   {
      this.tag = INT_TAG;
      this.valueI = value;
      // Implement C/C++ like semantics.
      this.valueB = (value != 0);
      valueL = new LinkedTree<Token>(new Token("null",0,0)); // empty tree
   }


   /**
      Construct a {@code Value} object that holds a boolean value.

      @param value  the boolean value for this {@code Value} object
   */
   public Value(boolean value)
   {
      this.tag = BOOL_TAG;
      this.valueB = value;
      // Implement C/C++ like semantics.
      this.valueI = (value ? 1 : 0);
      valueL = new LinkedTree<Token>(new Token("null",0,0)); // empty tree
   }


   /**
      Construct a Value object that holds a "lambda value".

      @param value  the abstract syntax {@link Tree} node of the function's lambda expression
   */
   public Value(Tree<Token> value)
   {
      this.tag = LAMBDA_TAG;
      this.valueL = value;
      this.valueB = true;
      this.valueI = 1;
   }


   @Override
   public String toString()
   {
      return "[tag->" + tag
                      + ", valueI->" + valueI
                      + ", valueB->" + valueB
                      + ", valueL->" + EvaluateAST_no_checking.toString(valueL)
                      + "]";
   }


   /**
      @return a simplified {@link String} representation of this {@code Value}
   */
   public String toSimpleString()
   {
      String result = "";

      if ( tag.equals(BOOL_TAG) )
      {
         result += valueB;
      }
      else if ( tag.equals(INT_TAG) )
      {
         result += valueI;
      }
      else if ( tag.equals(LAMBDA_TAG) )
      {
         result += valueL;
      }
      else // bad tag (shouldn't get here)
      {
         result += "[tag->" + tag
                            + ", valueI->" + valueI
                            + ", valueB->" + valueB
                            + ", valueL->" + EvaluateAST_no_checking .toString(valueL)
                            + "]";
      }

      return result;
   }
}
